@@ -1,4 +1,3 @@
-from rauc_handler import RaucHandler
 from base_service import BaseService
 from pathlib import Path
 from signalrcore.hub_connection_builder import HubConnectionBuilder
@@ -8,7 +7,6 @@ import asyncio
 import importlib
 import uuid
 import re
-import os
 import sys
 import traceback
 import socket
@@ -26,29 +24,46 @@ import utils
 import ssl
 ssl._create_default_https_context = ssl._create_unverified_context
 
-
 class microServiceBusHandler(BaseService):
     # region Constructor
     def __init__(self, id, queue):
         self.ready = False
+        self._connected = False
+        self._reconnect = False
+        self._missedheartbeat = 0
         self.base_uri = "https://microservicebus.com"
         home = str(Path.home())
         self.msb_dir = f"{os.environ['HOME']}/msb-py"
-
         self.printf(f"os.environ =  {os.environ}")
 
         self.service_path = f"{self.msb_dir}/services"
         self.msb_settings_path = f"{self.msb_dir}/settings.json"
-        self.rauc_handler = RaucHandler()
+        try:
+            from rauc_handler import RaucHandler
+            self.rauc_handler = RaucHandler()
+        except Exception as e:
+            self.printf("")
+            self.printf(f"Unable to initiate RAUC. Proceeding without RAUC support.")
+
         super(microServiceBusHandler, self).__init__(id, queue)
     # endregion
     # region Base functions
 
     async def Start(self):
         try:
-            await self.set_up_signalr()
+            
             self.settings = self.get_settings()
+            
+            if "hubUri" in self.settings:
+                self.printf("hubUri exists")
+                self.base_uri = self.settings["hubUri"]
+            else:
+                self.printf("hubUri not set")
+                self.settings["hubUri"] = self.base_uri
 
+            await self.Debug(f"instance: {self.base_uri}")
+
+            await self.set_up_signalr()
             # If no sas key, try provision using mac address
             sas_exists = "sas" in self.settings
             if(sas_exists == False):
@@ -66,7 +81,8 @@ class microServiceBusHandler(BaseService):
             await self.ThrowError(traceback.format_exc())
 
     async def _debug(self, message):
-        pass
+        if "nodeName" in self.settings:
+            self.connection.send("logMessage", [ self.settings["nodeName"], message.message[0], self.settings["organizationId"]])
 
     # endregion
     # region Private methods
@@ -82,17 +98,17 @@ class microServiceBusHandler(BaseService):
             os.mkdir(self.msb_dir)
 
         # Load settings file if exists
-        self.printf(self.msb_settings_path)
         if os.path.exists(self.msb_settings_path):
-            self.printf("Settings exists")
             with open(self.msb_settings_path) as f:
                 settings = json.load(f)
+
         else:
             self.printf("Creating settings")
             settings = {
                 "hubUri": self.base_uri
             }
             self.save_settings(settings)
+        
         return settings
 
     def save_settings(self, settings):
@@ -102,10 +118,10 @@ class microServiceBusHandler(BaseService):
     async def sign_in(self, settings, first_sign_in):
         if(first_sign_in == True):
             self.printf("Node created successfully")
+            settings["hubUri"] = self.settings["hubUri"] 
             self.settings = settings
             self.save_settings(settings)
 
-        await self.Debug("Signing in")
         await self.Debug("Node id: \033[95m" + settings["nodeName"] + "\033[0m")
         await self.Debug("Organization id:\033[95m " + settings["organizationId"] + "\033[0m")
         await self.Debug("Mac addresses:\033[95m " + ':'.join(re.findall('..', '%012x' % uuid.getnode())) + "\033[0m")
@@ -114,18 +130,18 @@ class microServiceBusHandler(BaseService):
             "id": "",
             "connectionId": "",
             "Name": settings["nodeName"],
-            "machineName": "",
+            "machineName": socket.gethostname(),
             "OrganizationID": settings["organizationId"],
             "npmVersion": "3.12.3",
             "sas": settings["sas"],
             "recoveredSignIn": "False",
-            # "ipAddresses": "",
             "macAddresses": [':'.join(re.findall('..', '%012x' % uuid.getnode()))]
         }
 
         # Use for debugging
-        self.connection.send("signIn", [hostData])
-        #self.connection.send("signInAsync", [hostData])
+        #self.connection.send("signIn", [hostData])
+        self.connection.send("signInAsync", [hostData])
+        await self.Debug("Signing in")
 
     def sign_in_sync(self, settings, first_sign_in):
         asyncio.run(self.sign_in(settings, first_sign_in))
@@ -133,7 +149,18 @@ class microServiceBusHandler(BaseService):
     async def create_node(self):
         mac = ':'.join(re.findall('..', '%012x' % uuid.getnode()))
         await self.Debug(mac)
-        self.connection.send("createNodeFromMacAddress", [mac])
+        request = {
+            "hostName" :socket.gethostname(),
+            "imei":None,
+            "ipAddress":"",
+            "macAddresses": mac,
+            "isModule":False
+        }
+        self.connection.send("assertNode", [request])
+        hurUri = self.settings["hubUri"]
+        await self.Debug("\033[93m Device information such as host name, MAC addresses and IP addresses\033[0m")
+        await self.Debug(f"\033[93m has been submitted to {hurUri} to pre-register the Node.\033[0m")
+        await self.Debug(f"\033[93m Please visit {hurUri} to claim this Node.\033[0m")
 
     def start_azure_iot_service(self, sign_in_response):
         try:
@@ -181,8 +208,10 @@ class microServiceBusHandler(BaseService):
     async def set_up_signalr(self):
         self.handler = logging.StreamHandler()
         self.handler.setLevel(logging.DEBUG)
+
         # Settings
         signalr_uri = f"{self.base_uri}/nodeHub"
+        await self.Debug(f"Connecting to {signalr_uri}")
         self.connection = HubConnectionBuilder()\
             .with_url(signalr_uri, options={"verify_ssl": False}) \
             .with_automatic_reconnect({
@@ -194,25 +223,21 @@ class microServiceBusHandler(BaseService):
         self.connection.keepAliveIntervalInMilliseconds = 1000 * 60 * 3
         self.connection.serverTimeoutInMilliseconds = 1000 * 60 * 6
         # Default listeners
-        self.connection.on_open(lambda: self.debug_sync(
-            "connection opened and handshake received ready to send messages"))
-        self.connection.on_close(lambda: self.debug_sync("connection closed"))
-        self.connection.on_error(lambda data: self.debug_sync(
-            f"An exception was thrown closed{data.error}"))
-
+        self.connection.on_open(lambda: self.connected())
+        self.connection.on_close(lambda: self.disconnected())
+        self.connection.on_error(lambda data: self.debug_sync(f"An exception was thrown closed{data.error}"))
+   
         # mSB.com listeners
-        self.connection.on(
-            "nodeCreated", lambda sign_in_info: self.sign_in_sync(sign_in_info[0], True))
-        self.connection.on(
-            "signInMessage", lambda sign_in_response: self.successful_sign_in(sign_in_response[0]))
-        self.connection.on(
-            "ping", lambda conn_id: self.ping_response(conn_id[0]))
+        self.connection.on("nodeCreated", lambda sign_in_info: self.sign_in_sync(sign_in_info[0], True))
+        self.connection.on("signInMessage", lambda sign_in_response: self.successful_sign_in(sign_in_response[0]))
+        self.connection.on("ping", lambda conn_id: self.ping_response(conn_id[0]))
         self.connection.on("errorMessage", lambda arg: self.debug_sync(arg[0]))
         self.connection.on("restart", lambda args: self.restart())
         self.connection.on("reboot", lambda args: self.reboot())
         self.connection.on("reset", lambda args: self.reset(args[0]))
-        self.connection.on("heartBeat", lambda messageList: self.printf(
-            "Heartbeat received: " + " ".join(messageList)))
+        self.connection.on("changeState", lambda args: self.change_state(args[0]))
+        self.connection.on("changeDebug", lambda args: self.change_debug(args[0]))
+        self.connection.on("heartBeat", lambda messageList: self.printf("Heartbeat received: " + " ".join(messageList)))
         self.connection.on("reportState", lambda id: self.report_state(id[0]))
         self.connection.on("updateFirmware", lambda firmware_response: self.update_firmware(
             firmware_response[0], firmware_response[1]))
@@ -220,10 +245,11 @@ class microServiceBusHandler(BaseService):
             boot_info[0], boot_info[1]))
         self.connection.on("getVpnSettingsResponse", lambda vpn_response: self.get_vpn_settings_response(
             vpn_response[0], vpn_response[1], vpn_response[2]))
-        self.connection.on(
-            "refreshVpnSettings", lambda response: self.refresh_vpn_settings_sync(response))
-        self.connection.on(
-            "sendMessage", lambda args: self.debug_sync(args[0]))
+        self.connection.on("refreshVpnSettings", lambda response: self.refresh_vpn_settings_sync(response))
+        self.connection.on("sendMessage", lambda args: self.debug_sync(args[0]))
+        self.connection.on("startTerminal", lambda args: self.start_terminal(args[0], args[1]))
+        self.connection.on("stopTerminal", lambda args: self.stop_terminal())
+        self.connection.on("terminalCommand", lambda args: self.terminal_command(args[0]))
 
         # region Not implemented event handlers
         self.connection.on(
@@ -232,11 +258,8 @@ class microServiceBusHandler(BaseService):
             "updateItinerary", lambda response: self.not_implemented("updateItinerary"))
         self.connection.on(
             "changeState", lambda response: self.not_implemented("changeState"))
-        self.connection.on(
-            "changeDebug", lambda response: self.not_implemented("changeDebug"))
         self.connection.on(
             "changeTracking", lambda response: self.not_implemented("changeTracking"))
-        #self.connection.on("sendMessage", lambda response: self.not_implemented("sendMessage"))
         self.connection.on(
             "forceUpdate", lambda response: self.not_implemented("forceUpdate"))
         self.connection.on(
@@ -291,12 +314,12 @@ class microServiceBusHandler(BaseService):
             "dockerComposeUp", lambda response: self.not_implemented("dockerComposeUp"))
         self.connection.on(
             "dockerComposeDown", lambda response: self.not_implemented("dockerComposeDown"))
-        self.connection.on(
-            "startTerminal", lambda response: self.not_implemented("startTerminal"))
-        self.connection.on(
-            "stopTerminal", lambda response: self.not_implemented("stopTerminal"))
-        self.connection.on(
-            "terminalCommand", lambda response: self.not_implemented("terminalCommand"))
+        # self.connection.on(
+        #     "startTerminal", lambda response: self.not_implemented("startTerminal"))
+        # self.connection.on(
+        #     "stopTerminal", lambda response: self.not_implemented("stopTerminal"))
+        # self.connection.on(
+        #     "terminalCommand", lambda response: self.not_implemented("terminalCommand"))
         self.connection.on(
             "downloadFile", lambda response: self.not_implemented("downloadFile"))
         self.connection.on(
@@ -308,26 +331,25 @@ class microServiceBusHandler(BaseService):
         self.set_interval(self.sendHeartbeat, 60 * 3)
     # endregion
     # region SignalR callback functions
-
     def successful_sign_in(self, sign_in_response):
         node_name = sign_in_response["nodeName"]
         tag_list = sign_in_response["tags"]
 
         asyncio.run(self.Debug(f"Node {node_name} signed in successfully"))
+        
+        if "hubUri" not in self.settings:
+            sign_in_response["hubUri"] = self.settings["hubUri"]
+        
         self.save_settings(sign_in_response)
-
         asyncio.run(self.SubmitAction("*", "msb_signed_in", {}))
-
-        #if sign_in_response['protocol'] == "AZUREIOT":
-        #    self.start_azure_iot_service(sign_in_response)
-
+        
         if os.path.isdir(self.service_path) == False:
             os.mkdir(self.service_path)
 
         if sign_in_response['itineraries'] is not None:
             for itinerary in sign_in_response['itineraries']:
                 pythonActivities = [srv for srv in itinerary["activities"]
-                                    if srv["userData"]["baseType"] == 'pythonfile']
+                                    if "baseType" in srv["userData"] and srv["userData"]["baseType"] == 'pythonfile']
                 for pythonActivity in pythonActivities:
 
                     host_config = [srv for srv in pythonActivity["userData"]
@@ -367,6 +389,8 @@ class microServiceBusHandler(BaseService):
                     asyncio.run(self.StartService(microService))
                     asyncio.run(self.Debug(f"Loading module {module_name}"))
 
+        settings = self.get_settings()
+        self.connection.send("signedIn", [ settings["nodeName"], socket.gethostname(), "Online", settings["organizationId"], False])
         self.sendHeartbeat()
 
     def not_implemented(self, event_handler):
@@ -374,10 +398,22 @@ class microServiceBusHandler(BaseService):
             f'SignalR event handler \033[93m"{event_handler}"\033[0m is not implemented in the Python Node'))
 
     def ping_response(self, conn_id):
-        self.printf("Ping response")
         settings = self.get_settings()
-        self.connection.send("pingResponse", [
-                             settings["nodeName"], socket.gethostname(), "Online", conn_id, False])
+        self.connection.send("pingResponse", [ settings["nodeName"], socket.gethostname(), "Online", conn_id, False])
+        asyncio.run(self.Debug("Ping response"))
+    
+    def connected(self):
+        self.debug_sync("Connection opened and handshake received ready to send messages")
+        self._connected = True
+
+        if(self._reconnect is True):
+            self.debug_sync("Reconnecting")
+            self.connection.send("reconnected", [self.settings["id"]])
+
+    def disconnected(self):
+        self.debug_sync("Connection closed")
+        self._connected = False
+        self._reconnect = True
 
     def sendHeartbeat(self):
         self.connection.send("heartBeat", ["echo"])
@@ -389,6 +425,27 @@ class microServiceBusHandler(BaseService):
             'notify', [id, f'Fetching environment state from {node_name}', 'INFO'])
 
         networks = []
+        ##xxx = psutil.net_if_stats()
+        for interface, snics in psutil.net_if_addrs().items():
+            for snic in snics:
+                if snic.family == socket.AF_INET:
+                    ni = {
+                        "name": interface,
+                        "ip_address":snic.address,
+                        "mac_address": utils.getHwAddr(interface),
+                        "netmask": snic.netmask,
+                        "type": ''
+                    }
+                    networks.append(ni)
+                else:
+                    self.printf("ignore")
+                    
+                    
+                    # if any(x["name"] == interface for x in networks) == False:
+                    #     networks.append(ni)
+                        
+        ##yyy = psutil.net_if_addrs().items()
+
         # interfaces = netifaces.interfaces()
         # for interface in interfaces:
         #     addrs = netifaces.ifaddresses(interface)
@@ -404,7 +461,11 @@ class microServiceBusHandler(BaseService):
         if_addrs = psutil.net_if_addrs()
         cpu_times = psutil.cpu_times()
         disk_info = psutil.disk_usage('/')
-        slot_status = self.rauc_handler.get_slot_status()
+        slot_status = None
+        try:
+            slot_status = self.rauc_handler.get_slot_status()
+        except Exception:
+            pass
         state = {
             "networks": networks,
             "memory": {
@@ -430,7 +491,7 @@ class microServiceBusHandler(BaseService):
             "raucState": slot_status
         }
         self.connection.send('reportStateResponse', [state, id])
-
+    
     def reset(self, id):
         asyncio.run(self.Debug("\033[93mResetting node\033[0m"))
         node_name = self.settings["nodeName"]
@@ -452,6 +513,17 @@ class microServiceBusHandler(BaseService):
         time.sleep(1)
         os.system("reboot")
 
+    def change_state(self, args):
+        self.settings["state"] = args
+        self.save_settings(self.settings)
+        asyncio.run(self.SubmitAction("*", "_change_state", args))
+
+    def change_debug(self, enableDebug):
+        self.debug_sync(f"Console debug {'enabled' if enableDebug else 'disabled'}")
+        self.settings["debug"] = enableDebug
+        self.save_settings(self.settings)
+        asyncio.run(self.SubmitAction("logger", "_change_debug", enableDebug))
+
     def update_firmware(self, force, connid):
         self.printf(force)
         self.printf(connid)
@@ -498,7 +570,7 @@ class microServiceBusHandler(BaseService):
             self.printf(f"Installing {file_name}")
 
             self.rauc_handler.install(file_name)
-
+    
     def set_boot_partition(self, partition, connid):
         self.printf(f"Marking partition {partition}")
         self.rauc_handler.mark_partition("active", partition)
@@ -518,14 +590,37 @@ class microServiceBusHandler(BaseService):
 
     def refresh_vpn_settings_sync(self, args):
         self.connection.send("getVpnSettings", [""])
-
+    
     def get_vpn_settings_response(self, vpnConfig, interfaceName, endpoint):
         message = {'vpnConfig': vpnConfig,
                    'interfaceName': interfaceName,
                    'endpoint': endpoint,
                    'vpnConfigPath': f"{self.msb_dir}/{interfaceName}.conf"}
 
-        asyncio.run(self.SubmitAction(
-            "vpnhelper", "get_vpn_settings_response", message))
+        asyncio.run(self.SubmitAction("vpnhelper", "get_vpn_settings_response", message))
+    
+    def start_terminal(self, connectionId, user):
+        message = {
+            'connectionId': connectionId,
+            'user': user
+        }
+        asyncio.run(self.SubmitAction("terminal", "_connect", message))
+    
+    def stop_terminal(self):
+       asyncio.run(self.SubmitAction("terminal", "_disconnect", {}))
+    
+    def terminal_command(self, args):
+        asyncio.run(self.SubmitAction("terminal", "_forward_command", args[0]))
 
     # endregion
+    # region Service callbacks
+    async def request_connectionstring(self, message):
+        action = message.message[0]["action"]     
+        await self.SubmitAction(message.source, action, self.settings["receiveConnectionString"])
+    async def terminal_output(self, message):
+        connectionId = message.message[0]["connectionId"]
+        data = message.message[0]["data"]    
+        self.connection.send("terminalData", [data, connectionId])
+    
+    # endregion
+    
\ No newline at end of file
